package test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.io.FilePermission;
import java.io.PrintWriter;
import java.io.StringWriter;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.openliberty.openaz.azapi.AzService;
import org.openliberty.openaz.azapi.constants.AzXacmlStrings;
import org.openliberty.openaz.pep.PepRequest;
import org.openliberty.openaz.pep.PepRequestFactory;
import org.openliberty.openaz.pep.PepResponse;
import org.openliberty.openaz.pep.PepResponseFactory;
import org.openliberty.openaz.pep.PepException;
import test.TestUtils;

public class BulkDecideTest {
	// use TestUtils for log obligations
	static TestUtils testUtils = new TestUtils();
	
    // variable defns to provide "context" for test program:
    // defn: "container": manages "environment" and "issues" env attributes
    public final static String CONTAINER = "orcl-weblogic"; // [a03]
    
    // defn: "application": manages "resources" and "issues" resource attrs
    public final static String APPLICATION = "hr-application-01"; // [a04]
    
    // test user; normally would be obtained from session info
    public final static String SAMPLE_SESSION_USER_NAME = "Joe User";
    public final static String SAMPLE_SESSION_AUTH_METHOD = "password";

    Log log = LogFactory.getLog(this.getClass());

    /**
     * @param args
     */
    public static void main(String[] args) throws Exception {
        // TODO Auto-generated method stub
		System.out.println("LogFactory = " + LogFactory.class);
    	BulkDecideTest bdt = new BulkDecideTest();
    	bdt.testBulkDecide();
    } // End of main()

    /**
     * Test the newPepBulkRequest options
     */
    public void testBulkDecide() {
        System.out.println("Log = " + log.getClass());

        AzService azService =
            new org.openliberty.openaz.pdp.provider.SimpleConcreteService();
        PepRequestFactory pep = new PepRequestFactory(CONTAINER, azService);
        
        // For bulk requests we do not want to throw exceptions
        // when processing the results.
        pep.getResponseFactory().setMissingAttributeBehavior(
        		PepResponseFactory.Behavior.RETURN_NO);
        pep.getResponseFactory().setProcessingErrorBehavior(
        		PepResponseFactory.Behavior.RETURN_NO);
        pep.getResponseFactory().setSyntaxErrorBehavior(
        		PepResponseFactory.Behavior.RETURN_NO);
        pep.getResponseFactory().setNotApplicableBehavior(
        		PepResponseFactory.Behavior.RETURN_NO);
        
        // An environment attribute
        Date now = new Date();

        // A collection of Subject attributes
        HashMap<String, String> subject = new HashMap<String, String>();
        subject.put(AzXacmlStrings.X_ATTR_SUBJECT_ID, "josh");
        subject.put(AzXacmlStrings.X_ATTR_SUBJECT_AUTHN_LOC_AUTHENTICATION_METHOD,
                    SAMPLE_SESSION_AUTH_METHOD);


        // An array of action strings that get submitted as list below
        String[] actions =
            new String[] { "read", "write", "delete", "read", "write",
                           "delete" };
        
        // Define 2 collections: one of resourceNames to go with the actions
        //  in the 4 parameter call below, and the other of Permissions using
        //  the same resourceNames and actions for each Permission, but for 
        //  use in the 3 parameter call
        String resourceName = "file:\\\\toplevel";
        ArrayList resources = new ArrayList();
        ArrayList resourceActions = new ArrayList();
        for (int i = 0; i < actions.length; i++) {
        	String resource = resourceName + "0" + new Integer(i);
            resources.add(resource);
            resourceActions.add(new FilePermission(resource, actions[i]));
        }

        // Try both the 4 param and 3 param bulk PepRequests
        try {
        	// first, try the 4 argument request w Strings
	        PepRequest request = pep.newBulkPepRequest(
	        		subject,
	        		Arrays.asList(actions),
	        		resources,
	        		now);
	        PepResponse response = null;
        	response = request.decide();
        	Iterator it = resources.iterator();
        	int i=0;
        	while (it.hasNext()){
                response.next();
        		i++;
        		Object obj = it.next();
                System.out.println(i + ". Action="+response.getAction()+
                		" Resource="+response.getResource()+
                		" allowed="+response.allowed());
                log.info(i + ". Action="+response.getAction()+
                		" Resource="+response.getResource()+
                		" allowed="+response.allowed());
                testUtils.logObligations(response);
        	}
        	// Try the 3 argument version w Permissions
        	request = pep.newBulkPepRequest(
	        		subject,
	        		resourceActions,
	        		now);
	        response = null;
        	response = request.decide();
        	it = resourceActions.iterator();
        	i=0;
        	while (it.hasNext()){
                response.next();
        		i++;
        		Object obj = it.next();
                System.out.println(i + ". Action="+response.getAction()+
                		" Resource="+response.getResource()+
                		" allowed="+response.allowed());
                log.info(i + ". Action="+response.getAction()+
                		" Resource="+response.getResource()+
                		" allowed="+response.allowed());
                testUtils.logObligations(response);
        	}
        }catch (PepException pepEx){
        	 StringWriter sw = new StringWriter();
        	 pepEx.printStackTrace(new PrintWriter(sw));
        	 log.warn("Caught Exception: " + pepEx + "\n" + sw);
        }
    }
}
