package test.objects;

import java.security.Permission;
import java.security.Principal;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openliberty.openaz.pep.JavaObjectMapper;
import org.openliberty.openaz.pep.AzWrapperRequestObject;
import org.openliberty.openaz.pep.AzWrapperObject;
import org.openliberty.openaz.pep.PepException;
import org.openliberty.openaz.azapi.constants.AzCategoryId;
import org.openliberty.openaz.azapi.constants.AzCategoryIdSubjectAccess;
import org.openliberty.openaz.azapi.constants.AzXacmlStrings;

import javax.security.auth.Subject;
import javax.security.auth.x500.X500Principal;

public class TestJaasSubjectMapper 
	implements JavaObjectMapper{
	Log log = LogFactory.getLog(this.getClass()); 
    public TestJaasSubjectMapper() {
        super();
    }
    /**
     * Returns the Set of Classes supported by this Mapper.
     * @return a set of classes
     */
    public Set<Class> getSupportedClasses(){
    	return supportedClasses;
    }
	private static Set<Class> supportedClasses =
		new HashSet<Class>() {{ add(Subject.class); }};
	public boolean canMapObject(Object obj){
    	log.info("canMap(Object obj): " + obj.getClass().getName());
        if (obj instanceof Subject){
        	log.info("obj instanceof Subject == true");
            return true;
        } else {
        	log.info("Cannot map obj instanceof: " + obj.getClass().getName());
            return false;
        }
	}
	public <T extends Enum<T> & AzCategoryId> 
		AzWrapperRequestObject<T> map(
				Object javaObject, 
				AzWrapperRequestObject<T> azWrapperRequestObject)
			throws PepException {
		log.info("Object to map: " + 
				javaObject.getClass().getName());
				if (javaObject instanceof Subject) {
					return convertSubject(
							(Subject)javaObject,
							azWrapperRequestObject);
				}		
				throw new PepException(
					"Can't map an object of class "+
					javaObject.getClass().getName());
	}
	
    private <T extends Enum<T> & AzCategoryId> 
	AzWrapperRequestObject<T > convertSubject(
				Subject jaasSubject,
				AzWrapperRequestObject<T> azWrapperRequestObject)
			throws PepException {
    	String attributeId = null;
    	if ( azWrapperRequestObject.wrappedObject.getAzCategoryId().
    			equals(AzCategoryIdSubjectAccess.AZ_CATEGORY_ID_SUBJECT_ACCESS)) {
    	    Set<Principal> xPrins = new HashSet<Principal>();
    	    xPrins = jaasSubject.getPrincipals();
    	    Iterator<? extends Principal> itPrin = xPrins.iterator();
    	    while (itPrin.hasNext()) {
    	    	Principal principal = itPrin.next();
    	    	if (principal instanceof X500Principal ) {
		    		attributeId = AzXacmlStrings.X_ATTR_SUBJECT_ID;
		    		azWrapperRequestObject.setAttribute(
		    				attributeId, 
		    				principal.getName());
    	    	}
    	    	else if (principal instanceof TestRolePrincipal) {
		    		attributeId = "test.role.principal.role-id";
		    		azWrapperRequestObject.setAttribute(
		    				attributeId,
		    				principal.getName());
	    	    }
    	    	else {
    	    		log.warn("Unknown principal cannot be mapped: " +
    	    				principal.getClass().getName());
    	    	}
    	    }
    	}
    	else {
    		throw new PepException(
    			"Can't convert JAAS Subject to wrapped object " +
    			" that is not Subject Category; This wrapped object " +
    			" has AzCategoryId: " +
    			azWrapperRequestObject.wrappedObject.getAzCategoryId() +
    			"and is of type: " + 
    			azWrapperRequestObject.wrappedObject.getClass().getName());
    	}
    	return azWrapperRequestObject;
    }

}
